@@ -42,6 +42,9 @@
 #include <glib/gstdio.h>
 #include <glib-object.h>
 
+#include <libxklavier/xklavier.h>
+#include <X11/Xlib.h> /* for Display */
+
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -595,14 +598,44 @@ get_default_language_name (GdmSessionDirect *session)
     return setlocale (LC_MESSAGES, NULL);
 }
 
+static char *
+get_system_default_layout (GdmSessionDirect *session)
+{
+    char *result;
+    Display *display;
+    
+    result = NULL;    
+    display = XOpenDisplay (session->priv->display_name);
+    if (display) {
+        XklConfigRec *config;
+        XklEngine *engine = xkl_engine_get_instance (display);
+        if (engine)
+        {
+            XklConfigRec *config = xkl_config_rec_new ();
+            if (xkl_config_rec_get_from_server (config, engine) && config->layouts && config->layouts[0]) {
+                    if (config->variants && config->variants[0] && config->variants[0][0])
+			    result = g_strdup_printf("%s\t%s", config->layouts[0], config->variants[0]);
+                    else
+			    result = g_strdup (config->layouts[0]);
+            }
+            g_object_unref (config);
+        }
+        XCloseDisplay (display);
+    }
+
+    if (!result)
+        result = g_strdup ("us");    
+    return result;
+}
+
 static const char *
 get_default_layout_name (GdmSessionDirect *session)
 {
-    if (session->priv->saved_layout != NULL) {
-                return session->priv->saved_layout;
+    if (!session->priv->saved_layout) {
+        session->priv->saved_layout = get_system_default_layout (session);
     }
 
-    return "us";
+    return session->priv->saved_layout;
 }
 
 static char *
@@ -1972,7 +2005,13 @@ gdm_session_direct_start_session (GdmSession *session)
         g_return_if_fail (impl->priv->is_running == FALSE);
 
         command = get_session_command (impl);
-        program = g_strdup_printf (GDMCONFDIR "/Xsession \"%s\"", command);
+
+        if (gdm_session_direct_bypasses_xsession (impl)) {
+                program = g_strdup (command);
+        } else {
+                program = g_strdup_printf (GDMCONFDIR "/Xsession \"%s\"", command);
+        }
+
         g_free (command);
 
         setup_session_environment (impl);
@@ -2062,6 +2101,55 @@ gdm_session_direct_get_username (GdmSessionDirect *session)
         return g_strdup (session->priv->selected_user);
 }
 
+gboolean
+gdm_session_direct_bypasses_xsession (GdmSessionDirect *session_direct)
+{
+        GError     *error;
+        GKeyFile   *key_file;
+        gboolean    res;
+        gboolean    bypasses_xsession = FALSE;
+        char       *filename;
+        char       *full_path;
+
+        g_return_val_if_fail (session_direct != NULL, FALSE);
+        g_return_val_if_fail (GDM_IS_SESSION_DIRECT (session_direct), FALSE);
+
+        filename = g_strdup_printf ("%s.desktop", get_session_name (session_direct));
+
+        key_file = g_key_file_new ();
+        error = NULL;
+        res = g_key_file_load_from_dirs (key_file,
+                                         filename,
+                                         get_system_session_dirs (),
+                                         &full_path,
+                                         G_KEY_FILE_NONE,
+                                         &error);
+        if (! res) {
+                g_debug ("GdmSessionDirect: File '%s' not found: %s", filename, error->message);
+                goto out;
+        }
+
+        error = NULL;
+        res = g_key_file_has_key (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-GDM-BypassXsession", NULL);
+        if (!res) {
+                goto out;
+        } else {
+                bypasses_xsession = g_key_file_get_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-GDM-BypassXsession", &error);
+                if (error) {
+                        bypasses_xsession = FALSE;
+                        g_error_free (error);
+                        goto out;
+                }
+                if (bypasses_xsession) {
+                        g_debug ("GdmSessionDirect: Session %s bypasses Xsession wrapper script", filename);
+                }
+        }
+
+out:
+        g_free (filename);
+        return bypasses_xsession;
+}
+
 static void
 gdm_session_direct_select_session (GdmSession *session,
                                    const char *text)
